/*****************************************************************************
*
* Copyright (C) 2016 Diwell Electronics Co.,Ltd.
* Project Name : CM1108 UART Code <softwareserial 이용>
* Version : 1.1 (2017.02.09)
* SYSTEM CLOCK : 16Mhz 
* BOARD : Arduino UNO. 5V operation 


 아두이노 PORT Description

1. RX : 13           
2. TX : 11
  CM1108 전원은 5V로 해야 하며 포트 연결 방법은 회로도를 참고하십시오.
  
 Revision history.

1. 2016.9.28  : First version is released.
2. 2017.2.09 : Fixed(Checksum calculation Error)
****************************************************************************/
#include <SoftwareSerial.h>

SoftwareSerial mySerial(13, 11);                     // RX 13, TX 11
unsigned char Send_data[4] = {0x11,0x01,0x01,0xED};    // Read Command
unsigned char Receive_Buff[8];                         // data buffer
unsigned char recv_cnt = 0;
unsigned int PPM_Value;                                // 농도(PPM) 저장 변수

void Send_CMD(void)                                    // COMMAND
{
  unsigned int i;
  for(i=0; i<4; i++)
  {
    mySerial.write(Send_data[i]);
    delay(1);
  }
}
unsigned char Checksum_cal(void)                       // CHECKSUM 
{
  unsigned char count, SUM=0;
  for(count=0; count<7; count++)
  {
     SUM += Receive_Buff[count];
  }
  return 256-SUM;
}

void setup() {
  pinMode(13,INPUT);
  pinMode(11,OUTPUT);
  Serial.begin(9600);
  while (!Serial) ;
  mySerial.begin(9600);
  while (!mySerial);
}


void loop() {
  Serial.print("Sending...");
  Send_CMD();  // Send Read Command
  while(1)
  {
    if(mySerial.available())
    { 
       Receive_Buff[recv_cnt++] = mySerial.read();
      if(recv_cnt ==8){recv_cnt = 0; break;}
    }
  } 
  
  if(Checksum_cal() == Receive_Buff[7])  // CS 확인을 통해 통신 에러 없으면
  {
     PPM_Value = Receive_Buff[3]<<8 | Receive_Buff[4];  // CALCULATE PPM
     Serial.write("   PPM : ");
     Serial.println(PPM_Value);
  }
  else
  {
    Serial.write("CHECKSUM Error");
  }
  delay(1000);        // 1000ms
}

